@@ -30,9 +30,12 @@ IN THE SOFTWARE.
 #include <rra/matchmaker.h>
 #include <dbus/dbus.h>
 #include <dbus/dbus-glib.h>
+#include <libunshield.h>
 
 #include "device-info.h"
 #include "sync-engine-glue.h"
+#include "synce_app_man.h"
+#include "utils.h"
 
 G_DEFINE_TYPE (WmDeviceInfo, wm_device_info, G_TYPE_OBJECT)
 
@@ -91,7 +94,19 @@ enum
   PSHMGR_STORETYPE_EXCH
 };
 
-/* partnership */
+enum
+{
+  APP_INDEX_COLUMN,
+  APP_NAME_COLUMN,
+  APP_N_COLUMNS
+};
+
+
+/*
+ **************************
+ partnership
+ **************************
+*/
 
 static void
 partners_create_button_clicked_rra_cb (GtkWidget *widget, gpointer data)
@@ -356,12 +371,12 @@ partners_create_button_clicked_synceng_cb (GtkWidget *widget, gpointer data)
   }
 
   gint response;
-  guint32 index, item;
+  guint32 item;
   gboolean active, result;
   guint id;
   gchar *name = NULL;
+  GtkListStore *store = NULL;
   GtkTreeIter iter;
-  GtkTreeModel *model;
   GtkCellRenderer *renderer;
   GtkTreeViewColumn *column;
   GtkWidget *device_info_dialog = priv->dialog;
@@ -369,7 +384,6 @@ partners_create_button_clicked_synceng_cb (GtkWidget *widget, gpointer data)
   GtkWidget *create_pship_dialog = glade_xml_get_widget (xml, "create_partnership_dialog"); 
   GtkWidget *pship_name_entry = glade_xml_get_widget (xml, "pship_name_entry");
   GtkWidget *sync_items_listview = glade_xml_get_widget (xml, "sync_items_listview");
-  GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (sync_items_listview));
   GArray *sync_items_required = NULL;
   GHashTable *sync_items = NULL;
   GError *error = NULL;
@@ -433,10 +447,10 @@ partners_create_button_clicked_synceng_cb (GtkWidget *widget, gpointer data)
     goto exit;
   }
 
-  GtkListStore *store = gtk_list_store_new (SYNCITEM_N_COLUMNS,
-                                            G_TYPE_INT,     /* index */
-                                            G_TYPE_BOOLEAN,     /* selected */
-                                            G_TYPE_STRING); /* program name */
+  store = gtk_list_store_new (SYNCITEM_N_COLUMNS,
+			      G_TYPE_INT,     /* index */
+			      G_TYPE_BOOLEAN,     /* selected */
+			      G_TYPE_STRING); /* program name */
 
   g_hash_table_foreach(sync_items, setup_sync_item_store, store);
   g_hash_table_destroy(sync_items);
@@ -529,6 +543,256 @@ exit:
 }
 
 
+#if GLIB_MINOR_VERSION < 14
+static void
+get_sync_item_keys(gpointer key,
+		   gpointer value,
+		   gpointer user_data)
+{
+  *(GList **)user_data = g_list_append(*(GList **)user_data, key);
+}
+#endif
+
+
+static gboolean
+check_delete_orphans(WmDeviceInfo *self, guint id, gchar *guid)
+{
+  WmDeviceInfoPrivate *priv = WM_DEVICE_INFO_GET_PRIVATE (self);
+  if (priv->disposed) {
+    g_warning("%s: Disposed object passed", G_STRFUNC);
+    return FALSE;
+  }
+
+  GError *error = NULL;
+  DBusGConnection *dbus_connection = NULL;
+  DBusGProxy *sync_engine_proxy = NULL;
+  gboolean result = FALSE;
+
+  GPtrArray* partnership_list = NULL;
+  GValueArray *partnership = NULL;
+  GHashTable *sync_items = NULL;
+  gboolean show_extra_warning;
+  GArray *orphaned_items = NULL;
+  GArray *sync_items_active = NULL;
+  GHashTable *pship_connections = NULL;
+  GList *item_keys = NULL, *tmp_list = NULL;
+  guint *sync_item_key = NULL;
+  guint index;
+
+  dbus_connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+  if (dbus_connection == NULL) {
+    g_critical("%s: Failed to open connection to bus: %s", G_STRFUNC, error->message);
+
+    GtkWidget *failed_dialog = gtk_message_dialog_new(GTK_WINDOW(priv->dialog),
+						      GTK_DIALOG_DESTROY_WITH_PARENT,
+						      GTK_MESSAGE_WARNING,
+						      GTK_BUTTONS_OK,
+						      "Deletion of partnership was unsuccessful: %s",
+						      error->message);
+
+    gtk_dialog_run(GTK_DIALOG(failed_dialog));
+    gtk_widget_destroy (failed_dialog);
+
+    g_error_free (error);
+    goto exit;
+  }
+
+  sync_engine_proxy = dbus_g_proxy_new_for_name (dbus_connection,
+						 "org.synce.SyncEngine",
+						 "/org/synce/SyncEngine",
+						 "org.synce.SyncEngine");
+  if (sync_engine_proxy == NULL) {
+    g_critical("%s: Failed to create proxy to sync engine", G_STRFUNC);
+
+    GtkWidget *failed_dialog = gtk_message_dialog_new(GTK_WINDOW(priv->dialog),
+						      GTK_DIALOG_DESTROY_WITH_PARENT,
+						      GTK_MESSAGE_WARNING,
+						      GTK_BUTTONS_OK,
+						      "Deletion of partnership was unsuccessful: Failed to connect to SyncEngine");
+
+    gtk_dialog_run(GTK_DIALOG(failed_dialog));
+    gtk_widget_destroy (failed_dialog);
+
+    goto exit;
+  }
+
+  result = org_synce_syncengine_get_partnerships (sync_engine_proxy, &partnership_list, &error);
+  if (!result) {
+    g_critical("%s: Error getting partnership list from sync-engine: %s", G_STRFUNC, error->message);
+    
+    GtkWidget *failed_dialog = gtk_message_dialog_new(GTK_WINDOW(priv->dialog),
+						      GTK_DIALOG_DESTROY_WITH_PARENT,
+						      GTK_MESSAGE_WARNING,
+						      GTK_BUTTONS_OK,
+						      "Failed to retrieve partnership information: %s",
+						      error->message);
+
+    gtk_dialog_run(GTK_DIALOG(failed_dialog));
+    gtk_widget_destroy (failed_dialog);
+
+    g_error_free(error);
+    goto exit;
+  }
+
+  result = org_synce_syncengine_get_item_types(sync_engine_proxy, &sync_items, &error);
+  if (!result) {
+    g_critical("%s: Error fetching sync item list: %s", G_STRFUNC, error->message);
+
+    GtkWidget *failed_dialog = gtk_message_dialog_new(GTK_WINDOW(priv->dialog),
+						      GTK_DIALOG_DESTROY_WITH_PARENT,
+						      GTK_MESSAGE_WARNING,
+						      GTK_BUTTONS_OK,
+						      "Failed to retrieve partnership information: %s",
+						      error->message);
+
+    gtk_dialog_run(GTK_DIALOG(failed_dialog));
+    gtk_widget_destroy (failed_dialog);
+
+    g_error_free(error);
+    goto exit;
+  }
+
+#if GLIB_MINOR_VERSION < 14
+  g_hash_table_foreach(sync_items,
+		       get_sync_item_keys,
+		       &item_keys);
+#else
+  item_keys = g_hash_table_get_keys(sync_items);
+#endif
+
+  pship_connections = g_hash_table_new_full(g_int_hash, g_int_equal, g_free, NULL);
+
+  tmp_list = item_keys;
+
+  while (tmp_list) {
+    sync_item_key = (guint *) g_malloc (sizeof (guint));
+    *sync_item_key = GPOINTER_TO_UINT(tmp_list->data);
+
+    g_hash_table_insert(pship_connections, sync_item_key, GUINT_TO_POINTER(0));
+
+    tmp_list = g_list_next(tmp_list);
+  }
+
+  g_list_free(item_keys);
+
+  index = 0;
+  while(index < partnership_list->len)
+    {
+      partnership = g_ptr_array_index(partnership_list, index);
+
+      /* an array of sync item ids - au - array of uint32 */
+      sync_items_active = g_value_get_boxed(g_value_array_get_nth (partnership, 6));
+
+      guint i;
+      guint active_item;
+      guint current_count;
+      for (i = 0; i < sync_items_active->len; i++) {
+	active_item = g_array_index(sync_items_active, guint, i);
+	current_count = GPOINTER_TO_UINT(g_hash_table_lookup(pship_connections, &active_item));
+	current_count++;
+
+	sync_item_key = (guint *) g_malloc (sizeof (guint));
+	*sync_item_key = active_item;
+
+	g_hash_table_insert(pship_connections, sync_item_key, GUINT_TO_POINTER(current_count));
+      }
+
+      index++;
+    }
+
+  show_extra_warning = FALSE;
+  orphaned_items = g_array_new(FALSE, TRUE, sizeof(guint));
+  index = 0;
+  while(index < partnership_list->len)
+    {
+      partnership = g_ptr_array_index(partnership_list, index);
+
+      guint partnership_id = g_value_get_uint(g_value_array_get_nth (partnership, 0));
+      const gchar *partnership_guid = g_value_get_string(g_value_array_get_nth (partnership, 1));
+
+      if ((partnership_id == id) && (strcmp(partnership_guid, guid) == 0)) {
+	sync_items_active = g_value_get_boxed(g_value_array_get_nth (partnership, 6));
+
+	guint i;
+	guint active_item;
+	guint current_count;
+	for (i = 0; i < sync_items_active->len; i++) {
+	  active_item = g_array_index(sync_items_active, guint, i);
+	  current_count = GPOINTER_TO_UINT(g_hash_table_lookup(pship_connections, &active_item));
+
+	  if (current_count == 1) {
+	    show_extra_warning = TRUE;
+	    g_array_append_val(orphaned_items, active_item);
+	  }
+	}
+
+	break;
+      }
+      index++;
+    }
+
+  g_hash_table_destroy(pship_connections);
+
+  if (show_extra_warning) {
+    GString *orphaned_str = NULL;
+    guint i;
+    guint item;
+    gint reply;
+
+    for (i = 0; i < orphaned_items->len; i++) {
+      item = g_array_index(orphaned_items, guint, i);
+     
+      if (!orphaned_str)
+	orphaned_str = g_string_new(g_hash_table_lookup(sync_items, GUINT_TO_POINTER(item)));
+      else
+	g_string_append_printf(orphaned_str, ", %s", (gchar *)g_hash_table_lookup(sync_items, GUINT_TO_POINTER(item)));
+    }
+
+    g_array_free(orphaned_items, TRUE);
+
+    GtkWidget *dialog = gtk_message_dialog_new(GTK_WINDOW(priv->dialog),
+					       GTK_DIALOG_DESTROY_WITH_PARENT,
+					       GTK_MESSAGE_WARNING,
+					       GTK_BUTTONS_YES_NO,
+					       "It appears you are using a Windows Mobile 6 device. Deleting this partnership might result in all items of types %s being deleted from the device because no other partnership is connected with them. Are you really sure you want to delete this partnership?",
+					       orphaned_str->str);
+
+    gtk_window_set_title(GTK_WINDOW(dialog), "Possible deletion of 'orphaned' items");
+    reply = gtk_dialog_run(GTK_DIALOG(dialog));
+    gtk_widget_destroy (dialog);
+
+    g_string_free(orphaned_str, TRUE);
+
+    if (reply != GTK_RESPONSE_YES)
+      result = FALSE;
+    else
+      result = TRUE;
+
+  } else
+    result = TRUE;
+
+exit:
+
+  if (partnership_list) {
+    index = 0;
+    while(index < partnership_list->len) {
+      partnership = g_ptr_array_index(partnership_list, index);
+      g_value_array_free(partnership);
+
+      index++;
+    }
+    g_ptr_array_free(partnership_list, TRUE);
+  }
+
+  if (sync_items) g_hash_table_destroy(sync_items);
+
+  if (sync_engine_proxy) g_object_unref(sync_engine_proxy);
+  if (dbus_connection) dbus_g_connection_unref(dbus_connection);
+
+  return result;
+}
+
+
 static void
 partners_remove_button_clicked_synceng_cb (GtkWidget *widget, gpointer data)
 {
@@ -544,9 +808,10 @@ partners_remove_button_clicked_synceng_cb (GtkWidget *widget, gpointer data)
   }
 
   guint index, id;
+  guint os_major, os_minor;
   gchar *guid = NULL;
   gchar *name;
-  GtkTreeIter iter;
+  GtkTreeIter iter, parent_iter;
   GtkTreeModel *model;
   GtkWidget *partners_list_view = glade_xml_get_widget (priv->xml, "partners_list");	
   GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (partners_list_view));
@@ -572,6 +837,20 @@ partners_remove_button_clicked_synceng_cb (GtkWidget *widget, gpointer data)
       return;
     }
 
+  /* if the user selected one of the subitems */
+
+  if (gtk_tree_model_iter_parent(model, &parent_iter, &iter)) {
+    g_free(guid);
+    g_free(name);
+    gtk_tree_model_get (model,
+			&parent_iter,
+			SYNCENG_INDEX_COLUMN, &index,
+			SYNCENG_ID_COLUMN, &id,
+			SYNCENG_GUID_COLUMN, &guid,
+			SYNCENG_NAME_COLUMN, &name,
+			-1);
+  }
+
   GtkWidget *device_info_dialog = priv->dialog;
   GtkWidget *confirm_dialog = gtk_message_dialog_new(GTK_WINDOW(device_info_dialog),
 						     GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -628,6 +907,21 @@ partners_remove_button_clicked_synceng_cb (GtkWidget *widget, gpointer data)
     goto exit;
   }
 
+  /*
+    Check whether the removal of this particular partnership would result
+    in orphaned itemtypes, that would be deleted by wm6
+  */
+
+  g_object_get(priv->device,
+	       "os-major", &os_major,
+	       "os-minor", &os_minor,
+	       NULL);
+
+  if ((os_major > 4) && (os_minor > 1)) {
+    if (!check_delete_orphans(self, id, guid))
+      goto exit;
+  }
+
   result = org_synce_syncengine_delete_partnership(sync_engine_proxy, id, guid, &error);
   if (!result) {
     g_critical("%s: Error deleting partnership via sync-engine: %s", G_STRFUNC, error->message);
@@ -673,7 +967,7 @@ partners_selection_changed_synceng_cb (GtkTreeSelection *selection, gpointer dat
   GtkTreeModel *model;
   gint index, id;
   gchar *name;
-  GtkWidget *partners_create_button, *partners_remove_button;
+  GtkWidget *partners_remove_button;
 
   partners_remove_button = glade_xml_get_widget (priv->xml, "partners_remove_button");	
 
@@ -701,15 +995,6 @@ sync_item_sort(gconstpointer a, gconstpointer b)
 }
 
 
-#if GLIB_MINOR_VERSION < 14
-get_sync_item_keys(gpointer key,
-		   gpointer value,
-		   gpointer user_data)
-{
-  *(GList **)user_data = g_list_append(*(GList **)user_data, key);
-}
-#endif
-
 static void
 partners_setup_view_store_synceng(WmDeviceInfo *self)
 {
@@ -1192,7 +1477,11 @@ partners_setup_view(WmDeviceInfo *self)
 }
 
 
-/* system info */
+/*
+ ***********************
+ system info
+ ***********************
+*/
 
 /* from librapi2 pstatus */
 static const char*
@@ -1325,6 +1614,9 @@ system_info_setup_view_store(WmDeviceInfo *self)
 
     g_free(storage_size);
     g_free(ram_size);
+  } else {
+    gtk_label_set_text(GTK_LABEL(sys_info_store_ram), "N/A");
+    gtk_label_set_text(GTK_LABEL(sys_info_store_storage), "N/A");
   }
 }
 
@@ -1368,7 +1660,7 @@ system_info_setup_view(WmDeviceInfo *self)
   version.dwOSVersionInfoSize = sizeof(version);
 
   if (CeGetVersionEx(&version)) {
-    char *details = wstr_to_current(version.szCSDVersion);
+    char *details = wstr_to_utf8(version.szCSDVersion);
     char *platform = NULL;
 
     if (VER_PLATFORM_WIN32_CE == version.dwPlatformId)
@@ -1435,7 +1727,11 @@ system_info_setup_view(WmDeviceInfo *self)
 }
 
 
-/* power */
+/*
+ **************
+ power
+ **************
+*/
 
 static const gchar*
 get_ACLineStatus_string(unsigned ACLineStatus)
@@ -1571,6 +1867,328 @@ system_power_setup_view(WmDeviceInfo *self)
 }
 
 
+/*
+ ****************
+ applications
+ ****************
+ */
+
+static void
+progress_bar_pulse(gpointer data)
+{
+  gtk_progress_bar_pulse(GTK_PROGRESS_BAR(data));
+  while (gtk_events_pending()) {
+    gtk_main_iteration_do(FALSE);
+  }
+}
+
+GtkWidget *
+busy_window_new(const gchar *title, const gchar *message, GtkWidget **progressbar)
+{
+  GtkWidget *window, *hbox, *image, *vbox, *label;
+
+  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_title(GTK_WINDOW(window), title);
+  gtk_container_set_border_width(GTK_CONTAINER(window), 14);
+
+  hbox = gtk_hbox_new(FALSE, 10);
+  gtk_container_add(GTK_CONTAINER(window), hbox);
+
+  image = gtk_image_new_from_stock(GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_DIALOG);
+  gtk_container_add(GTK_CONTAINER(hbox), image);
+
+  vbox = gtk_vbox_new(FALSE, 10);
+  gtk_container_add(GTK_CONTAINER(hbox), vbox);
+
+  label = gtk_label_new(message);
+  gtk_container_add(GTK_CONTAINER(vbox), label);
+
+  *progressbar = gtk_progress_bar_new();
+  gtk_container_add(GTK_CONTAINER(vbox), *progressbar);
+
+  return window;
+}
+
+static void
+applications_selection_changed (GtkTreeSelection *selection, gpointer user_data) 
+{
+  WmDeviceInfo *self = WM_DEVICE_INFO(user_data);
+  if (!self) {
+    g_warning("%s: Invalid object passed", G_STRFUNC);
+    return;
+  }
+  WmDeviceInfoPrivate *priv = WM_DEVICE_INFO_GET_PRIVATE (self);
+  if (priv->disposed) {
+    g_warning("%s: Disposed object passed", G_STRFUNC);
+    return;
+  }
+
+  GtkTreeIter iter;
+  GtkTreeModel *model;
+  gint number;
+  gchar *program;
+
+  GtkWidget *app_remove_button = glade_xml_get_widget(priv->xml, "app_remove_button");
+
+  if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
+    gtk_tree_model_get (model, &iter,
+			APP_NAME_COLUMN, &program,
+			APP_INDEX_COLUMN, &number,
+			-1);
+    g_debug("%s: selected %d: %s", G_STRFUNC, number, program);
+    g_free (program);
+    gtk_widget_set_sensitive(app_remove_button, TRUE);
+  } else {
+    gtk_widget_set_sensitive(app_remove_button, FALSE);
+  }
+}
+
+
+static void
+applications_setup_view_store(WmDeviceInfo *self) 
+{
+  if (!self) {
+    g_warning("%s: Invalid object passed", G_STRFUNC);
+    return;
+  }
+  WmDeviceInfoPrivate *priv = WM_DEVICE_INFO_GET_PRIVATE (self);
+  if (priv->disposed) {
+    g_warning("%s: Disposed object passed", G_STRFUNC);
+    return;
+  }
+
+  GtkWidget *app_treeview = glade_xml_get_widget(priv->xml, "applications_treeview");
+  GtkWidget *remove_button = glade_xml_get_widget(priv->xml, "app_remove_button");
+
+  GtkListStore *store = gtk_list_store_new (APP_N_COLUMNS,
+					    G_TYPE_INT,     /* index */
+					    G_TYPE_STRING); /* program name */
+  GtkTreeIter iter;
+  GtkWidget *fetchwindow, *progressbar;
+  GList *programlist = NULL;
+  GList *tmplist = NULL;
+  int i = 0;
+  GError *error = NULL;
+  gboolean result;
+
+  fetchwindow = busy_window_new(_("Fetching information..."), _("Fetching the list of installed applications from the PDA."), &progressbar);
+
+  gtk_progress_bar_pulse(GTK_PROGRESS_BAR(progressbar));
+  gtk_widget_show_all(fetchwindow);
+
+  while (gtk_events_pending()) {
+    gtk_main_iteration_do(FALSE);
+  }
+
+  result = synce_app_man_create_program_list(&programlist, progress_bar_pulse, progressbar, &error);
+  if (!result) {
+    synce_error_dialog(_("Couldn't fetch the list of applications from the device: %s"), error->message);
+    g_error_free(error);
+  }
+  gtk_widget_destroy(fetchwindow);
+
+  tmplist = programlist;
+
+  /* i is not really used except to insert a sequence number in the list */
+  i = 0;
+  while (tmplist != NULL) {
+    gtk_list_store_append (store, &iter);  /* Acquire an iterator */
+
+    gtk_list_store_set (store, &iter,
+			APP_INDEX_COLUMN, i,
+			APP_NAME_COLUMN, tmplist->data,
+			-1);
+
+    tmplist = g_list_next(tmplist);    
+    i++;
+  }
+
+  gtk_tree_view_set_model (GTK_TREE_VIEW(app_treeview), GTK_TREE_MODEL (store));
+  g_object_unref (G_OBJECT (store));
+
+  tmplist = programlist;
+  while (tmplist != NULL) {
+    g_free(tmplist->data);
+    tmplist = g_list_next(tmplist);    
+  }
+  g_list_free(programlist);
+
+  gtk_widget_set_sensitive(remove_button, FALSE);
+}
+
+
+static void
+on_app_add_button_clicked(GtkButton *button, gpointer user_data)
+{
+  WmDeviceInfo *self = WM_DEVICE_INFO(user_data);
+  if (!self) {
+    g_warning("%s: Invalid object passed", G_STRFUNC);
+    return;
+  }
+  WmDeviceInfoPrivate *priv = WM_DEVICE_INFO_GET_PRIVATE (self);
+  if (priv->disposed) {
+    g_warning("%s: Disposed object passed", G_STRFUNC);
+    return;
+  }
+
+  GtkWidget *installdialog;
+  GtkWidget *fetchwindow, *progressbar;
+  gchar *filepath;
+  gint response;
+  GError *error = NULL;
+  gchar *message = NULL;
+  gboolean result;
+
+  installdialog = gtk_file_chooser_dialog_new (_("Select installation file"),
+					       NULL,
+					       GTK_FILE_CHOOSER_ACTION_OPEN,
+					       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					       GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+					       NULL);
+  response = gtk_dialog_run (GTK_DIALOG (installdialog));
+  filepath = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(installdialog));
+  gtk_widget_destroy (installdialog);
+
+  if (response == GTK_RESPONSE_ACCEPT) {
+    message = g_strdup_printf(_("Installing from file \"%s\"..."), filepath);
+    fetchwindow = busy_window_new(_("Installing Software..."), message, &progressbar);
+    g_free(message);
+
+    gtk_progress_bar_pulse(GTK_PROGRESS_BAR(progressbar));
+    gtk_widget_show_all(fetchwindow);
+
+    while (gtk_events_pending()) {
+      gtk_main_iteration_do(FALSE);
+    }
+
+    g_debug("%s: requested installation of file %s", G_STRFUNC, filepath);
+    result = synce_app_man_install(filepath, progress_bar_pulse, progressbar, &error);
+
+    gtk_widget_hide_all(fetchwindow);
+    gtk_widget_destroy(fetchwindow);
+    if (!result) {
+      synce_error_dialog(_("Failed to install from file %s: %s"), filepath, error->message);
+      g_error_free(error);
+    } else {
+      synce_info_dialog(_("Successfully started installation of file \"%s\". Check your device to see if any additional steps are required. The program list must be manually refreshed after installation has completed."), filepath);
+    }
+  }
+
+  g_free(filepath);
+}
+
+void
+on_app_remove_button_clicked(GtkButton *button, gpointer user_data)
+{
+  WmDeviceInfo *self = WM_DEVICE_INFO(user_data);
+  if (!self) {
+    g_warning("%s: Invalid object passed", G_STRFUNC);
+    return;
+  }
+  WmDeviceInfoPrivate *priv = WM_DEVICE_INFO_GET_PRIVATE (self);
+  if (priv->disposed) {
+    g_warning("%s: Disposed object passed", G_STRFUNC);
+    return;
+  }
+
+  GtkWidget *app_treeview = glade_xml_get_widget(priv->xml, "applications_treeview");
+  GtkTreeIter iter;
+  GtkTreeModel *model;
+  GtkTreeSelection *selection;
+  gint number;
+  gchar *program = NULL;
+  GError *error = NULL;
+
+  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(app_treeview));
+  if (gtk_tree_selection_get_selected (selection, &model, &iter))
+    {
+      gtk_tree_model_get (model, &iter,
+			  APP_NAME_COLUMN, &program,
+			  APP_INDEX_COLUMN, &number,
+			  -1);
+
+      if (synce_app_man_uninstall(program, &error)) {
+	g_debug("%s: successfully removed program %s", G_STRFUNC, program);
+	synce_info_dialog(_("The program \"%s\" was successfully removed."), program);
+	applications_setup_view_store(self);
+      } else {
+	g_warning("%s: failed to remove program %s: %s", G_STRFUNC, program, error->message);
+	synce_error_dialog(_("The program \"%s\" could not be removed: %s"), program, error->message);
+	g_error_free(error);
+      }
+      g_free (program);
+    }
+}
+
+
+static void
+applications_setup_view(WmDeviceInfo *self)
+{
+  if (!self) {
+    g_warning("%s: Invalid object passed", G_STRFUNC);
+    return;
+  }
+  WmDeviceInfoPrivate *priv = WM_DEVICE_INFO_GET_PRIVATE (self);
+  if (priv->disposed) {
+    g_warning("%s: Disposed object passed", G_STRFUNC);
+    return;
+  }
+
+  wm_device_rapi_select(priv->device);
+
+  GtkCellRenderer *renderer;
+  GtkTreeViewColumn *column;
+  GtkTreeSelection *selection;
+  GtkWidget *app_treeview, *app_add_button, *app_remove_button;
+
+  app_treeview = glade_xml_get_widget(priv->xml, "applications_treeview");
+  app_add_button = glade_xml_get_widget(priv->xml, "app_add_button");
+  app_remove_button = glade_xml_get_widget(priv->xml, "app_remove_button");
+
+  g_signal_connect(G_OBJECT(app_add_button), "clicked",
+		   G_CALLBACK(on_app_add_button_clicked),
+		   self);
+
+  g_signal_connect(G_OBJECT(app_remove_button), "clicked",
+		   G_CALLBACK(on_app_remove_button_clicked),
+		   self);
+
+  unshield_set_log_level(UNSHIELD_LOG_LEVEL_WARNING);
+
+  applications_setup_view_store(self) ;
+
+  column = gtk_tree_view_column_new ();
+  renderer = gtk_cell_renderer_text_new ();
+  gtk_tree_view_column_pack_start (column, renderer, TRUE);
+  gtk_tree_view_column_set_attributes (column, renderer,
+				       "text", APP_INDEX_COLUMN, NULL);
+
+  gtk_tree_view_column_set_visible(GTK_TREE_VIEW_COLUMN(column), FALSE);
+  gtk_tree_view_append_column (GTK_TREE_VIEW(app_treeview), column);
+
+  column = gtk_tree_view_column_new ();
+  gtk_tree_view_column_set_title (GTK_TREE_VIEW_COLUMN(column),"Application");
+  renderer = gtk_cell_renderer_text_new ();
+  gtk_tree_view_column_pack_start (column, renderer, TRUE);
+  gtk_tree_view_column_set_attributes (column, renderer,
+				       "text", APP_NAME_COLUMN, NULL);
+
+  gtk_tree_view_append_column (GTK_TREE_VIEW(app_treeview), column);
+
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (app_treeview));
+  gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
+  g_signal_connect (G_OBJECT (selection), "changed",
+		    G_CALLBACK (applications_selection_changed), self);
+
+  gtk_widget_show (app_treeview);
+}
+
+/*
+ *************
+ general
+ *************
+ */
+
 static void
 device_info_refresh_button_clicked_cb (GtkWidget *widget, gpointer data)
 {
@@ -1595,6 +2213,7 @@ device_info_refresh_button_clicked_cb (GtkWidget *widget, gpointer data)
     partners_setup_view_store_rra(self);
   }
 
+  applications_setup_view_store(self);
   system_info_setup_view_store(self);
   system_power_setup_view(self);
 }
@@ -1652,6 +2271,7 @@ device_info_setup_dialog (WmDeviceInfo *self)
   g_free(title);
 
   partners_setup_view(self);
+  applications_setup_view(self);
   system_info_setup_view(self);
   system_power_setup_view(self);
 
@@ -1765,9 +2385,6 @@ wm_device_info_dispose (GObject *obj)
 static void
 wm_device_info_finalize (GObject *obj)
 {
-  WmDeviceInfo *self = WM_DEVICE_INFO(obj);
-  WmDeviceInfoPrivate *priv = WM_DEVICE_INFO_GET_PRIVATE (self);
-
   if (G_OBJECT_CLASS (wm_device_info_parent_class)->finalize)
     G_OBJECT_CLASS (wm_device_info_parent_class)->finalize (obj);
 }
@@ -1793,7 +2410,6 @@ wm_device_info_class_init (WmDeviceInfoClass *klass)
 						     NULL, NULL,
 						     g_cclosure_marshal_VOID__VOID,
 						     G_TYPE_NONE, 0);
-  klass->signals[DEVICE_INFO_LAST_SIGNAL] = 0;
 
   param_spec = g_param_spec_pointer ("device", "Device",
                                      "The device object",
