@@ -30,11 +30,18 @@ IN THE SOFTWARE.
 #include <string.h>
 
 #include "device.h"
+#include "utils.h"
 
 G_DEFINE_TYPE (WmDevice, wm_device, G_TYPE_OBJECT)
 
 typedef struct _WmDevicePrivate WmDevicePrivate;
 struct _WmDevicePrivate {
+  /* identifier from dccm */
+  gchar *object_name;
+  gchar *dccm_type;
+
+  WmDeviceStatus connection_status;
+
   /* lowercase name from info file */
   gchar *name;
 
@@ -67,7 +74,10 @@ struct _WmDevicePrivate {
 /* properties */
 enum
   {
-    PROP_NAME = 1,
+    PROP_OBJECT_NAME = 1,
+    PROP_DCCM_TYPE,
+    PROP_CONNECTION_STATUS,
+    PROP_NAME,
     PROP_OS_MAJOR,
     PROP_OS_MINOR,
     PROP_BUILD_NUMBER,
@@ -91,6 +101,60 @@ enum
 
      /* methods */
 
+gboolean
+wm_device_rapi_connect(WmDevice *self)
+{
+        if (!self) {
+                g_warning("%s: Invalid object passed", G_STRFUNC);
+                return FALSE;
+        }
+        WmDevicePrivate *priv = WM_DEVICE_GET_PRIVATE (self);
+
+        if (priv->disposed) {
+                g_warning("%s: Disposed object passed", G_STRFUNC);
+                return FALSE;
+        }
+
+        if (priv->connection_status != DEVICE_STATUS_CONNECTED) {
+                g_warning("%s: device not fully connected", G_STRFUNC);
+                return FALSE;
+        }
+
+        HRESULT hr;
+        RapiConnection *rapi_conn;
+
+        if (priv->rapi_conn)
+                return TRUE;
+
+        g_debug("%s: Initialising device rapi connection", G_STRFUNC);
+
+        rapi_conn = rapi_connection_from_name(priv->name);
+        if (!rapi_conn) {
+                g_critical("%s: Failed to obtain rapi connection to %s", G_STRFUNC, priv->name);
+                return FALSE;
+        }
+
+        rapi_connection_select(rapi_conn);
+
+        hr = CeRapiInit();
+        if (FAILED(hr)) {
+                g_critical("%s: Failed to initialise rapi connection to %s: %d: %s", G_STRFUNC, priv->name, hr, synce_strerror(hr));
+                rapi_connection_destroy(rapi_conn);
+                return FALSE;
+        }
+
+        priv->rapi_conn = rapi_conn;
+
+        priv->device_name = get_device_name_via_rapi();
+
+        if (!(priv->device_name)) {
+                CeRapiUninit();
+                rapi_connection_destroy(rapi_conn);
+                return FALSE;
+        }
+        return TRUE;
+}
+
 void
 wm_device_rapi_select(WmDevice *self)
 {
@@ -105,6 +169,11 @@ wm_device_rapi_select(WmDevice *self)
     return;
   }
 
+  if (priv->connection_status != DEVICE_STATUS_CONNECTED) {
+    g_warning("%s: device not fully connected", G_STRFUNC);
+    return;
+  }
+
   rapi_connection_select(priv->rapi_conn);
   return;
 }
@@ -225,6 +294,11 @@ wm_device_get_os_version_impl(WmDevice *self)
     return 0;
   }
 
+  if (priv->connection_status != DEVICE_STATUS_CONNECTED) {
+    g_warning("%s: device not fully connected", G_STRFUNC);
+    return 0;
+  }
+
   return priv->os_major;
 }
 
@@ -242,6 +316,11 @@ wm_device_get_build_number_impl(WmDevice *self)
     return 0;
   }
 
+  if (priv->connection_status != DEVICE_STATUS_CONNECTED) {
+    g_warning("%s: device not fully connected", G_STRFUNC);
+    return 0;
+  }
+
   return priv->build_number;
 }
 
@@ -259,6 +338,11 @@ wm_device_get_processor_type_impl(WmDevice *self)
     return 0;
   }
 
+  if (priv->connection_status != DEVICE_STATUS_CONNECTED) {
+    g_warning("%s: device not fully connected", G_STRFUNC);
+    return 0;
+  }
+
   return priv->processor_type;
 }
 
@@ -276,6 +360,11 @@ wm_device_get_partner_id_1_impl(WmDevice *self)
     return 0;
   }
 
+  if (priv->connection_status != DEVICE_STATUS_CONNECTED) {
+    g_warning("%s: device not fully connected", G_STRFUNC);
+    return 0;
+  }
+
   return priv->partner_id_1;
 }
 
@@ -293,6 +382,11 @@ wm_device_get_partner_id_2_impl(WmDevice *self)
     return 0;
   }
 
+  if (priv->connection_status != DEVICE_STATUS_CONNECTED) {
+    g_warning("%s: device not fully connected", G_STRFUNC);
+    return 0;
+  }
+
   return priv->partner_id_2;
 }
 
@@ -327,6 +421,11 @@ wm_device_get_class_impl(WmDevice *self)
     return NULL;
   }
 
+  if (priv->connection_status != DEVICE_STATUS_CONNECTED) {
+    g_warning("%s: device not fully connected", G_STRFUNC);
+    return NULL;
+  }
+
   return g_strdup(priv->class);
 }
 
@@ -344,6 +443,11 @@ wm_device_get_hardware_impl(WmDevice *self)
     return NULL;
   }
 
+  if (priv->connection_status != DEVICE_STATUS_CONNECTED) {
+    g_warning("%s: device not fully connected", G_STRFUNC);
+    return NULL;
+  }
+
   return g_strdup(priv->hardware);
 }
 
@@ -361,6 +465,11 @@ wm_device_get_device_name_impl(WmDevice *self)
     return NULL;
   }
 
+  if (priv->connection_status != DEVICE_STATUS_CONNECTED) {
+    g_warning("%s: device not fully connected", G_STRFUNC);
+    return NULL;
+  }
+
   return g_strdup(priv->device_name);
 }
 
@@ -378,6 +487,11 @@ wm_device_get_password_impl(WmDevice *self)
     return NULL;
   }
 
+  if (priv->connection_status != DEVICE_STATUS_CONNECTED) {
+    g_warning("%s: device not fully connected", G_STRFUNC);
+    return NULL;
+  }
+
   return g_strdup(priv->password);
 }
 
@@ -395,6 +509,11 @@ wm_device_get_key_impl(WmDevice *self)
     return 0;
   }
 
+  if (priv->connection_status != DEVICE_STATUS_CONNECTED) {
+    g_warning("%s: device not fully connected", G_STRFUNC);
+    return 0;
+  }
+
   return priv->key;
 }
 
@@ -412,6 +531,11 @@ wm_device_get_dccm_pid_impl(WmDevice *self)
     return 0;
   }
 
+  if (priv->connection_status != DEVICE_STATUS_CONNECTED) {
+    g_warning("%s: device not fully connected", G_STRFUNC);
+    return 0;
+  }
+
   return priv->dccm_pid;
 }
 
@@ -429,6 +553,11 @@ wm_device_get_ip_impl(WmDevice *self)
     return NULL;
   }
 
+  if (priv->connection_status != DEVICE_STATUS_CONNECTED) {
+    g_warning("%s: device not fully connected", G_STRFUNC);
+    return NULL;
+  }
+
   return g_strdup(priv->ip);
 }
 
@@ -446,6 +575,11 @@ wm_device_get_transport_impl(WmDevice *self)
     return NULL;
   }
 
+  if (priv->connection_status != DEVICE_STATUS_CONNECTED) {
+    g_warning("%s: device not fully connected", G_STRFUNC);
+    return NULL;
+  }
+
   return g_strdup(priv->transport);
 }
 
@@ -463,6 +597,11 @@ wm_device_get_port_impl(WmDevice *self)
     return NULL;
   }
 
+  if (priv->connection_status != DEVICE_STATUS_CONNECTED) {
+    g_warning("%s: device not fully connected", G_STRFUNC);
+    return NULL;
+  }
+
   return g_strdup(priv->port);
 }
 
@@ -502,6 +641,11 @@ wm_device_get_power_status_impl(WmDevice *self)
     return NULL;
   }
 
+  if (priv->connection_status != DEVICE_STATUS_CONNECTED) {
+    g_warning("%s: device not fully connected", G_STRFUNC);
+    return NULL;
+  }
+
   wm_device_rapi_select(self);
 
   memset(&power, 0, sizeof(SYSTEM_POWER_STATUS_EX));
@@ -535,6 +679,11 @@ wm_device_get_store_status_impl(WmDevice *self)
     return NULL;
   }
 
+  if (priv->connection_status != DEVICE_STATUS_CONNECTED) {
+    g_warning("%s: device not fully connected", G_STRFUNC);
+    return NULL;
+  }
+
   wm_device_rapi_select(self);
 
   memset(&store, 0, sizeof(store));
@@ -559,6 +708,9 @@ wm_device_init(WmDevice *self)
 {
   WmDevicePrivate *priv = WM_DEVICE_GET_PRIVATE (self);
 
+  priv->object_name = NULL;
+  priv->dccm_type = NULL;
+  priv->connection_status = DEVICE_STATUS_UNKNOWN;
   priv->os_major = 0;
   priv->os_minor = 0;
   priv->build_number = 0;
@@ -590,6 +742,15 @@ wm_device_get_property (GObject    *obj,
 
   switch (property_id) {
 
+  case PROP_OBJECT_NAME:
+    g_value_set_string (value, priv->object_name);
+    break;
+  case PROP_DCCM_TYPE:
+    g_value_set_string (value, priv->dccm_type);
+    break;
+  case PROP_CONNECTION_STATUS:
+    g_value_set_uint (value, priv->connection_status);
+    break;
   case PROP_NAME:
     g_value_set_string (value, priv->name);
     break;
@@ -659,6 +820,17 @@ wm_device_set_property (GObject      *obj,
   WmDevicePrivate *priv = WM_DEVICE_GET_PRIVATE (self);
 
   switch (property_id) {
+  case PROP_OBJECT_NAME:
+    g_free (priv->object_name);
+    priv->object_name = g_value_dup_string (value);
+    break;
+  case PROP_DCCM_TYPE:
+    g_free (priv->dccm_type);
+    priv->dccm_type = g_value_dup_string (value);
+    break;
+  case PROP_CONNECTION_STATUS:
+    priv->connection_status = g_value_get_uint (value);
+    break;
   case PROP_NAME:
     g_free (priv->name);
     priv->name = g_value_dup_string (value);
@@ -757,6 +929,8 @@ wm_device_finalize (GObject *obj)
   WmDevice *self = WM_DEVICE(obj);
   WmDevicePrivate *priv = WM_DEVICE_GET_PRIVATE (self);
 
+  g_free(priv->object_name);
+  g_free(priv->dccm_type);
   g_free(priv->name);
   g_free(priv->class);
   g_free(priv->hardware);
@@ -802,6 +976,30 @@ wm_device_class_init (WmDeviceClass *klass)
   klass->wm_device_get_power_status = &wm_device_get_power_status_impl;
   klass->wm_device_get_store_status = &wm_device_get_store_status_impl;
 
+  param_spec = g_param_spec_string ("object-name", "Object Name",
+                                    "The device's dccm specific object name.",
+                                    NULL,
+                                    G_PARAM_READWRITE |
+                                    G_PARAM_STATIC_NICK |
+                                    G_PARAM_STATIC_BLURB);
+  g_object_class_install_property (gobject_class, PROP_OBJECT_NAME, param_spec);
+
+  param_spec = g_param_spec_string ("dccm-type", "Dccm type",
+                                    "Which dccm type the device is connected to.",
+                                    NULL,
+                                    G_PARAM_READWRITE |
+                                    G_PARAM_STATIC_NICK |
+                                    G_PARAM_STATIC_BLURB);
+  g_object_class_install_property (gobject_class, PROP_DCCM_TYPE, param_spec);
+
+  param_spec = g_param_spec_uint ("connection-status", "Connection status",
+                                  "The state of the connection to the device.",
+                                  0, G_MAXUINT32, 0,
+                                  G_PARAM_READWRITE |
+                                  G_PARAM_STATIC_NICK |
+                                  G_PARAM_STATIC_BLURB);
+  g_object_class_install_property (gobject_class, PROP_CONNECTION_STATUS, param_spec);
+
   param_spec = g_param_spec_string ("name", "Device name",
                                     "The device's name.",
                                     NULL,
